-- MainWindow.lua
-- Originally TheOneBagWindow.lua written by Rod, extended by Habna


import "HabnaPlugins.HugeBag.settingsWindow"; -- LUA settings file
import "HabnaPlugins.HugeBag.functionsWindow"; -- LUA functions file
import "HabnaPlugins.HugeBag.optionsWindow"; -- LUA options file


local resources = {
	  Item = { Background = "HabnaPlugins/HugeBag/Resources/slotBackground.tga" },
	  Resize = { Background = "HabnaPlugins/HugeBag/Resources/resize.tga" }
};

LoadSettings();

if ShowTitle then
	frmMain = class( Turbine.UI.Lotro.Window ); --Window with skin & title
	--frmMain = class( Turbine.UI.Lotro.GoldWindow ); --Gold Window
else
	frmMain = class( Turbine.UI.Window ); --Window with no skin & no title
end

function frmMain:Constructor()
	--v SETTINGS SECTION v--
	if ShowTitle then
		Turbine.UI.Lotro.Window.Constructor( self ); --Window with skin & title
		--Turbine.UI.Lotro.GoldWindow.Constructor( self ); --Gold Window
	else
		Turbine.UI.Window.Constructor( self ); --Window with no skin no title
	end
	
	self.cursor = nil;

	--**v Set HugeBag window v**
	self:SetSize( WinWidth, WinHeight );
	self:SetPosition( WinLocX, WinLocY );
	self:SetWantsKeyEvents( true );
	self:SetVisible( StartVisible );
	if zOrder then self:SetZOrder( 1 );	end --1 Always on top / 0 Not always on top
	self:Activate();
	--**^
	
	Turbine.PluginManager.RefreshAvailablePlugins();
	loaded_plugins = Turbine.PluginManager.GetLoadedPlugins();

	HBLBChecker = Turbine.UI.Control();
	HBLBChecker:SetWantsUpdates( true );
	
	HBLBChecker.Update = function( sender, args )
		for k,v in pairs(loaded_plugins) do
			if v.Name == "HugeBag LootBug" then
				Turbine.PluginManager.UnloadScriptState( 'HugeBagLootBug' );
				self:SetVisible( WasVisible );
				--write("Bug detected, HugeBag was reloaded!");
				break;
			end
		end
		HBLBChecker:SetWantsUpdates( false );
		WasVisible = self:IsVisible();
		SaveSettings();
	end

	--**v Skin v**
	HBSkin = Turbine.UI.Control();
	HBSkin:SetParent( self );
	HBSkin:SetPosition( 0, 0 );
	HBSkin:SetSize( WinWidth, WinHeight );
	HBSkin:SetBackColor( none );
	HBSkin:SetVisible( not ShowSkin );

	HBSkin.MouseMove = function( sender, args )
		self.MouseMove( sender, args );
	end

	HBSkin.MouseDown = function( sender, args )
		self.MouseDown( sender, args );
	end

	HBSkin.MouseUp = function( sender, args )
		self.MouseUp( sender, args );
	end

	HBSkin.MouseLeave = function( sender, args )
		self.MouseLeave( sender, args );
	end
	--**^
	--**v Lock Label v**
	LockLabel = Turbine.UI.Control();
	LockLabel:SetParent( self );
	LockLabel:SetPosition( 23, 0 );
	LockLabel:SetSize( WinWidth - 50, 40 );
	LockLabel:SetVisible( true );
	LockLabel.Enable = not LockPosition;

	LockLabel.MouseClick = function( sender, args )
		if ( args.Button == Turbine.UI.MouseButton.Right ) then ReloadHugeBag(); end
	end
	
	LockLabel.MouseMove = function( sender, args )
		CornerClick = "Title";
		self.MouseMove( sender, args );
	end
	
	LockLabel.MouseDown = function( sender, args )
		if not LockPosition then
			CornerClick = "Title";
			self.MouseDown( sender, args );
		end
	end

	LockLabel.MouseUp = function( sender, args )
		self.MouseUp( sender, args );
	end

	LockLabel.MouseLeave = function( sender, args )
		CornerClick = "";
		self.MouseLeave( sender, args );
	end
	--**^
	-- **v Sort button v**
	SortButton = Turbine.UI.Button();
	SortButton:SetParent( self );
	SortButton:SetText( "Sort" );
	SortButton:SetSize( 45, 12 );
	SortButton:SetForeColor( grey );
	SortButton:SetOutlineColor( black );
	SortButton:SetFontStyle( Turbine.UI.FontStyle.Outline );
	SortButton:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	SortButton:SetVisible( ShowButton );

	SortButton.MouseClick = function(sender, args)
		if ( args.Button == Turbine.UI.MouseButton.Left ) then
			Sort();
		else
			FromButton = "Sort";
			if ( SortButton:GetLeft() == MidPos ) then buttonmenu:ShowMenu(); else Mix(); end
		end
	end

	SortButton.MouseHover = function(sender, args)
		SortButton:SetForeColor( green );
	end

	SortButton.MouseLeave = function(sender, args)
		SortButton:SetForeColor( grey );
	end
	-- **^
	-- **v Search button v**
	SearchButton = Turbine.UI.Button();
	SearchButton:SetParent( self );
	SearchButton:SetText( "Search" );
	SearchButton:SetSize( 45, 12 );
	SearchButton:SetForeColor( grey );
	SearchButton:SetOutlineColor( black );
	SearchButton:SetFontStyle( Turbine.UI.FontStyle.Outline );
	SearchButton:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	SearchButton:SetVisible( ShowButton );

	SearchButton.MouseClick = function(sender, args)
		if ( args.Button == Turbine.UI.MouseButton.Left ) then
			Searching();
		else
			FromButton = "Search";
			if ( SearchButton:GetLeft() == MidPos ) then buttonmenu:ShowMenu(); end
		end
	end

	SearchButton.MouseHover = function(sender, args)
		SearchButton:SetForeColor( green );
	end

	SearchButton.MouseLeave = function(sender, args)
		SearchButton:SetForeColor( grey );
	end
	--**^
	--**v Merge button on bottom bar v**
	MergeButton = Turbine.UI.Button();
	MergeButton:SetParent( self );
	MergeButton:SetText( "Merge" );
	MergeButton:SetSize( 45, 12 );
	MergeButton:SetOutlineColor( black );
	MergeButton:SetForeColor( grey );
	MergeButton:SetFontStyle( Turbine.UI.FontStyle.Outline );
	MergeButton:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	MergeButton:SetVisible( ShowButton );

	MergeButton.MouseClick = function(sender, args)
		if ( args.Button == Turbine.UI.MouseButton.Left ) then
			Merge();
		else
			FromButton = "Merge";
			if ( MergeButton:GetLeft() == MidPos ) then buttonmenu:ShowMenu(); end
		end
	end

	MergeButton.MouseHover = function(sender, args)
		MergeButton:SetForeColor( green );
	end

	MergeButton.MouseLeave = function(sender, args)
		MergeButton:SetForeColor( grey );
	end
	--**^
	-- **v Set the itemlistbox v**
	self.itemListBox = Turbine.UI.ListBox();
	self.itemListBox:SetParent( self );
	--self.itemListBox:SetReverseFill( ReverseFill );
	self.itemListBox:SetOrientation( Turbine.UI.Orientation.Vertical );
	if HorizontalOrientation then self.itemListBox:SetOrientation( Turbine.UI.Orientation.Horizontal ); end
	self.itemListBox:SetBackColor( Turbine.UI.Color( bcAlpha, bcRed, bcGreen, bcBlue ) );
	self.itemListBox:SetAllowDrop( true );

	self.itemListBox.DragDrop = function( sender, args )
		--write("drag/drop");
		local shortcut = args.DragDropInfo:GetShortcut();
		local destinationItemControl = self.itemListBox:GetItemAt( args.X, args.Y );
		
		if ( destinationItemControl ~= nil ) then -- ** if nil, then it's out of range, so don't do anything **
			local destinationIndex = self.itemListBox:IndexOfItem( destinationItemControl );
			self.backpack:PerformShortcutDrop(shortcut, destinationIndex, Turbine.UI.Control.IsShiftKeyDown() );
		end
	end
	-- **^
	-- **v Set the scrollbar v**
	self.itemListBoxScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.itemListBoxScrollBar:SetParent( self );	
	-- **^
	-- **v Search box v**
	SearchInput = Turbine.UI.Lotro.TextBox();
	SearchInput:SetParent( self );
	SearchInput.Text = "";
	SearchInput:SetSize( 150, 20 );
	SearchInput:SetMultiline( false );
	SearchInput:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 );
	SearchInput:SetText( SearchInput.Text );
	SearchInput:SetVisible( false );
	
	SearchInput.FocusGained = function()
		SearchInput:SetWantsUpdates( true );
	end
	
	SearchInput.FocusLost = function()
		SearchInput:SetWantsUpdates( false );
		SearchInput:SetText( "" );
		SearchInput:SetVisible( false );
		if ShowButton then
			SortButton:SetVisible( true );
			SearchButton:SetVisible( true );
			MergeButton:SetVisible( true );
			Search();
			AllVisible = true;
		end
	end
	
	SearchInput.Update = function()
		Search();
	end
	-- **^
	-- **v Get player gameplay v**
	if pcall( Turbine.Gameplay.LocalPlayer.GetInstance ) then
		self.backpack = Turbine.Gameplay.LocalPlayer.GetInstance():GetBackpack();
	else
		self.backpack = Turbine.Gameplay.LocalPlayer():GetBackpack();
	end
	-- **^

	backpackSize = self.backpack:GetSize();
	self:SetBackPack(); --> ** Set the backpack **
	
	self.MouseDown = function( sender, args )
		if ( args.Button == Turbine.UI.MouseButton.Left ) then
			if ( args.X < 23 and args.Y > 15 and args.Y < 50 )then
				CornerClick = "TopLeft";
			elseif ( ( args.X < 23 ) and ( args.Y > ( self:GetHeight() - 30 ) ) ) then
				CornerClick = "BottomLeft";
			elseif ( args.X > ( self:GetWidth() -30 ) ) and ( args.Y > ( self:GetHeight() - 30 ) ) then
				CornerClick = "BottomRight";
			elseif ( args.X > 50 ) and ( args.X < ( self:GetWidth() - 50 ) ) and ( args.Y > ( self:GetHeight() - 12 ) ) then
				CornerClick = "BottomBorder";
			elseif ( args.X < 6 ) and ( args.Y > 60 ) and ( args.Y < ( self:GetHeight() - 45 ) ) then
				CornerClick = "LeftBorder";
			elseif ( args.X > ( self:GetWidth() - 6 ) ) and ( args.Y > 60 ) and ( args.Y < ( self:GetHeight() - 45 ) ) then
				CornerClick = "RightBorder";
			end
			ClicX, ClicY = args.X + 23, args.Y;
			dragStartX = args.X;
			dragStartY = args.Y;
			dragging = true;
		elseif ( args.Button == Turbine.UI.MouseButton.Right ) then
			if ( args.X < 23 and args.Y > 15 and args.Y < 50 )then 
				--TopLeft
				frmOptions();
			elseif ( ( args.X < 23 ) and ( args.Y > ( self:GetHeight() - 30 ) ) ) then
				--BottomLeft
				if self:GetLeft() >= 204 then
					itemsmenu:ShowMenuAt(self:GetLeft() - 200, self:GetTop());
				else
					itemsmenu:ShowMenuAt(self:GetLeft() + self:GetWidth() + 5, self:GetTop());
				end
			end
		end
	end
	
	self.MouseUp = function( sender, args )
		dragging = false;
		OutOfBound = false;
		CornerClick = "";
		SaveSettings();
	end
	
	self.MouseMove = function( sender, args )
		if ( dragging ) then
			self.MouseLeave( sender, args );
			WinWidth, WinHeight = self:GetSize();
			WinLocX, WinLocY = self:GetPosition();
			local screenWidth = Turbine.UI.Display:GetWidth();
			local screenHeight = Turbine.UI.Display:GetHeight();
			local tempY = WinHeight + ( args.Y - dragStartY );

			if CornerClick == "BottomRight" then
				if not LockSize then
					-- **v Check horizontal v**
					local tempX = WinWidth + ( args.X - dragStartX );
					if not ButtonAtLeft then
						if ( self:GetWidth() >= 218 ) then
							ButtonsLeft();
							ButtonsRight();
						end
					end
					-- Prevent window to go out of bound (Right side)
					if ( WinLocX + tempX ) > screenWidth then
						self:SetSize( ( screenWidth - WinLocX ), WinHeight );
					else
						self:SetSize( tempX, tempY );
						dragStartX = dragStartX + (args.X - dragStartX);
						dragStartY = dragStartY + (args.Y - dragStartY);
					end
					-- **^
				end
			elseif CornerClick == "BottomLeft" then
				if not LockSize then
					-- **v Check horizontal v**
					local tempX = WinWidth - ( args.X - dragStartX );
					if not ButtonAtLeft then
						if ( self:GetWidth() >= 218 ) then
							ButtonsLeft();
							ButtonsRight();
						end
					end
					--Prevent window to go out of bound (Left side)
					if WinLocX < 0 then
						dragging = false;
						self:SetPosition( 0, WinLocY );
					else
						self:SetPosition( WinLocX + ( args.X - dragStartX ), WinLocY );
						self:SetSize( tempX, tempY );
						dragStartY = dragStartY + ( args.Y - dragStartY );
					end
					-- **^
				end
			elseif CornerClick == "Title" then
				local mouseX, mouseY = Turbine.UI.Display.GetMousePosition();
				self:SetPosition( mouseX - ClicX, mouseY - ClicY);
			elseif CornerClick == "BottomBorder" then
				if not LockSize then
					if tempY < 125 then -- Minimum height
						self:SetHeight( 125 );
					else
						self:SetHeight( tempY );
					end
					dragStartY = dragStartY + (args.Y - dragStartY);
				end
			elseif CornerClick == "LeftBorder" then
				if not LockSize then
					local tempX = WinWidth - ( args.X - dragStartX );
					if not ButtonAtLeft then
						if ( self:GetWidth() >= 218 ) then
							ButtonsLeft();
							ButtonsRight();
						end
					end
					if WinLocX < 0 then --Prevent window to go out of bound (Left side)
						dragging = false;
						self:SetPosition( 0, WinLocY );
					elseif self:GetWidth() <= 218 then
						dragging = false;
						self:SetSize( 219, WinHeight );
					else
						self:SetPosition( WinLocX + ( args.X - dragStartX ), WinLocY );
						self:SetSize( tempX, WinHeight );
					end
				end
			elseif CornerClick == "RightBorder" then
				if not LockSize then
					local tempX = WinWidth + ( args.X - dragStartX );
					if not ButtonAtLeft then
						if ( self:GetWidth() >= 218 ) then
							ButtonsLeft();
							ButtonsRight();
						end
					end
					if ( WinLocX + tempX ) > screenWidth then -- Prevent window to go out of bound (Right side)
						self:SetSize( ( screenWidth - WinLocX ), WinHeight );
					else
						self:SetSize( tempX, WinHeight );
						dragStartX = dragStartX + (args.X - dragStartX);
					end
				end
			end
			--[[ Need to go out of bound when no skin on HugeBag
			-- **v Check vertical v**
			-- Prevent window to go out of bound (Bottom)
			if ( WinLocY + tempY ) > screenHeight then
				self:SetSize( WinWidth, ( screenHeight - WinLocY ) );
			end
			-- **^
			]]
			self:PerformLayout();
		else
			if CornerClick == "Title" then
				ShowLotroResizeCursor();
			else
				if ( args.X < 23 and args.Y > 15 and args.Y < 50 )then
					-- "TopLeft"
					self.cursor = rightclicCursor;
					local mouseX, mouseY = Turbine.UI.Display.GetMousePosition();
					self.cursor:SetPosition( mouseX - self.cursor.xOffset, mouseY - self.cursor.yOffset);
					self.cursor:SetVisible( true );
				elseif ( ( args.X < 23 ) and ( args.Y > ( self:GetHeight() - 30 ) ) ) then
					-- "BottomLeft"
					ShowResizeRightClicCursor();
				elseif ( args.X > ( self:GetWidth() - 30 ) ) and ( args.Y > ( self:GetHeight() - 30 ) ) then
					-- "BottomRight"
					ShowResizeCursor();
				elseif ( args.X > 50 ) and ( args.X < ( self:GetWidth() - 50 ) ) and ( args.Y > ( self:GetHeight() - 12 ) ) then
					-- Bottom border
					ShowUpDownCursor();
				elseif ( args.X < 6 ) and ( args.Y > 60 ) and ( args.Y < ( self:GetHeight() - 45 ) ) then
					-- Left border
					ShowLeftRightCursor();
				elseif ( args.X > ( self:GetWidth() - 6 ) ) and ( args.Y > 60 ) and ( args.Y < ( self:GetHeight() - 45 ) ) then
					--Right border
					ShowLeftRightCursor();
				else
					self.MouseLeave( sender, args );
				end
			end
		end
		self:PrintTitle();
	end

	self.MouseLeave = function( sender, args )
		if self.cursor ~= nil then
			self.cursor:SetVisible( false );
			self.cursor = nil;
		end
	end
	
	self.Closing = function( sender, args )
		SaveSettings();
	end
	
	self.PositionChanged = function(args)
		WinLocX, WinLocY = self:GetPosition()
		SaveSettings();
	end
	
	self.KeyDown = function( sender, args )
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then -- **v Hide HugeBag if ESC key is press v**
			if ( HideWEsc ) then
				self:SetVisible( false );
				WasVisible = false;
				SaveSettings();
			end
		end

		if ( args.Action == 268435635 ) then -- **v Hide HugeBag if F12 key is press v**
			if self:IsVisible() then self:SetVisible( false ); end
			F12Press = not F12Press;
			if not F12Press then self:SetVisible( WasVisible ); end
		end
		
		-- **v Show HugeBag if any key bound is press v**
		if ( args.Action == Turbine.UI.Lotro.Action.ToggleBags or
		     args.Action == Turbine.UI.Lotro.Action.ToggleBag1 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag2 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag3 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag4 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag5 )
		then
			if not F12Press then -- **v show HugeBag only if F12 key was not press v**
				self:SetVisible( not self:IsVisible() )
				WasVisible = self:IsVisible();
				SaveSettings();
			end
		end
	end
	
	self.backpack.SizeChanged = function( sender, args )
		self:SetBackPack();
	end
	
	self.backpack.ItemAdded = function( sender, args )
		--write("item added to index " .. args.Index);
		bAdd = true;
		AddTime = Turbine.Engine.GetGameTime();

		self.items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
		freeslots = freeslots - 1;
		self:PrintTitle();
	end
	
	self.backpack.ItemRemoved = function( sender, args )
		--write("item removed from index " .. args.Index);
		--write("bAdd: " .. tostring(bAdd) .. " IsSorting: " .. tostring(IsSorting) .. " IsMixing: " .. tostring(IsMixing));
		if bAdd and not IsSorting and not IsMixing then
			RemTime = Turbine.Engine.GetGameTime();
			elapsed = RemTime - AddTime;
			--write("elapsed: " .. tostring(elapsed));
			if elapsed == 0 then bGo = true; end
		end
		bAdd = false;

		self.items[args.Index]:SetItem( nil );
		freeslots = freeslots + 1;
		self:PrintTitle();

		if bGo then ReloadHugeBag(); end
	end
	
	self.backpack.ItemMoved = function( sender, args )
		--write("item moved from index " .. args.OldIndex .. " to " .. args.NewIndex);
		self.items[args.OldIndex]:SetItem( self.backpack:GetItem( args.OldIndex ) );
		self.items[args.NewIndex]:SetItem( self.backpack:GetItem( args.NewIndex ) );
	end
end

--**v Set the backpack v**
function frmMain:SetBackPack()
	self.items = nil;
	itemsCtl = nil;
	itemsLbl = nil;
	self.itemListBox:ClearItems();
	freeslots = 0;

	self.items = {};
	itemsCtl = {};
	itemsLbl = {};

	for i = 1, backpackSize do
		-- Item background
		itemsCtl[i] = Turbine.UI.Control();
		itemsCtl[i]:SetSize( 35, 35 );
		itemsCtl[i]:SetBackground( resources.Item.Background );
		itemsCtl[i]:SetBlendMode( Turbine.UI.BlendMode.Overlay );

		-- Slot num in background
		itemsLbl[i] = Turbine.UI.Label();
		itemsLbl[i]:SetParent( itemsCtl[i] );
		itemsLbl[i]:SetSize( 15, 15 );
		itemsLbl[i]:SetPosition( 10, 10 );
		itemsLbl[i]:SetText( i );
		itemsLbl[i]:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
		itemsLbl[i]:SetForeColor( verylightgrey );
		itemsLbl[i]:SetBlendMode( Turbine.UI.BlendMode.Overlay );

		-- Item
		self.items[i] = Turbine.UI.Lotro.ItemControl( self.backpack:GetItem( i ) );
		self.items[i]:SetParent( itemsCtl[i] );

		if ( self.backpack:GetItem( i ) == nil ) then freeslots = freeslots + 1; end

		self.itemListBox:AddItem( itemsCtl[i] );
	end

	self:PerformLayout();
	self:PrintTitle()
end
-- **^
-- **v Layout of the items in the listbox v**
function frmMain:PerformLayout()
	WinWidth, WinHeight = self:GetSize();
	WinLocX, WinLocY = self:GetPosition();
	itemWidth, itemHeight = self.items[1]:GetSize();
	
	listWidth = WinWidth - 37;
	listHeight = WinHeight - 52;

	self.itemListBox:SetSize( listWidth, listHeight );
	self.itemListBox:SetPosition( InitXPos, InitYPos );

	self.itemListBox:SetOrientation( Turbine.UI.Orientation.Vertical );
	if HorizontalOrientation then self.itemListBox:SetOrientation( Turbine.UI.Orientation.Horizontal ); end
	
	if self.itemListBox:GetOrientation() == 0 then -- 0 => Horizontal /  1=> Vertical
		itemsPerRow = listWidth / itemWidth;
		--if ReverseFill then
			
		--else
			self.itemListBoxScrollBar:SetOrientation( Turbine.UI.Orientation.Vertical );
			self.itemListBox:SetVerticalScrollBar( self.itemListBoxScrollBar );

			self.itemListBoxScrollBar:SetPosition( WinWidth - 20, 40 );
			self.itemListBoxScrollBar:SetSize( 10, listHeight - 15 );
		--end
	else
		itemsPerRow = listHeight / itemHeight;
		--if ReverseFill then

		--else
			self.itemListBoxScrollBar:SetOrientation( Turbine.UI.Orientation.Horizontal );
			self.itemListBox:SetHorizontalScrollBar( self.itemListBoxScrollBar );

			self.itemListBoxScrollBar:SetPosition( self.itemListBox:GetLeft(), WinHeight - 23);
			self.itemListBoxScrollBar:SetSize( listWidth, 10 );
		--end
	end
	
	self.itemListBox:SetMaxItemsPerLine( itemsPerRow );
	
	if ( self:GetWidth() <= 218 ) then self:SetSize( 218, self:GetHeight()); end -- Limit to minimum width

	LockLabel:SetSize(WinWidth - 50, 33 ); --> ** Resize the LockLabel with HugeBag width **
	HBSkin:SetSize( WinWidth, WinHeight ); --> ** Resize the HBSkin with HugeBag width an height **

	if ( ButtonAtTop ) then
		TopPos = 23;
	else
		TopPos = WinHeight - 16;
	end
	SortButton:SetPosition( SortLeft,  TopPos ); --> ** Replace 'Sort' button with HugeBag height **
	SearchButton:SetPosition( SearchLeft,  TopPos ); --> ** Replace 'Search' button with HugeBag height **
	MergeButton:SetPosition( MergeLeft, TopPos ); --> ** Replace 'Merge' button with HugeBag height **
	SearchInput:SetPosition( SortLeft,  TopPos - 7 ); --> ** Replace 'Search Box' with HugeBag height **
end
-- **^
-- **v Print the title with the free/used/max info v**
function frmMain:PrintTitle()
	if ( ShowSlotsInfo ) then
		local usedslots = backpackSize - freeslots;
		if WinWidth < 262 then
			self:SetText( "F: " .. freeslots .. "  U: " .. usedslots .. "  M:" .. backpackSize );
		elseif WinWidth < 310 then
			self:SetText( "HB -- F: " .. freeslots .. "  U: " .. usedslots .. "  M:" .. backpackSize );
		elseif WinWidth < 380 then
			self:SetText( "HugeBag -- F: " .. freeslots .. "  U: " .. usedslots .. "  M:" .. backpackSize );
		else
			self:SetText( "HugeBag -- Free: " .. freeslots .. "  Used: " .. usedslots .. "  Max:" .. backpackSize );
		end
	else
		if WinWidth < 165 then
			self:SetText( "HB" );
		else
			self:SetText( "HugeBag" );
		end
	end
end
-- **^